require 'digest/sha1'
require 'fileutils'
require 'yaml'
require File.dirname(__FILE__) + '/logline.rb'

module XhochY

  module Drossellog
   
    # Parses a hole Apache combined log file
    class LogFileParser
      # The name of the logfile which is processed with a certain instance
      attr_reader :filename
      # The SHA1 checksum of the logfile
      attr_reader :sha1sum
    
      # Create a new XYLogFileParser instance
      def initialize(filename, tmp_path)
        @filename = filename
        @sha1sum = ''
        @tmp_path = tmp_path
      end
      
      # Generate the SHA1 checksum of the log file
      def generate_sha1sum
        s = File.read(@filename)
        @sha1sum = Digest::SHA1.hexdigest(s) 
      end
      
      # Checks if a directory already exists and if the content is outdated
      def outdated?
        if @sha1sum.empty?
          generate_sha1sum
        end
        
        path = File.join(@tmp_path, File.basename(@filename));
        path2 = File.join(path, 'sha1sum')
        
        if File.exist?(path) # look for tmp-directory
          
          if File.exist?(path2) # look for sha1sum file
            sum = File.read(path2).strip
            return !sum.eql?(@sha1sum)
          else # sha1sum file does not exist
            return true
          end
          
        else # tmp-directory does not exist
          return true
        end
      end #^ outdated?
      
      # Remove the old, outdated data
      def cleanup_old
        path = File.join(@tmp_path, File.basename(@filename));
        
        # remove old directory
        if File.exist?(path)
          FileUtils.rm_r(path)
          puts path
        end
        
        # make new
        FileUtils.mkdir_p(path)
      end
      
      # Parse the complete file and store the Lines in an array.
      # If a line could not be parsed(most times it is not in the
      # correct format) it will be outputted to stdout. If you think
      # the line was well formed check the regular expression in
      # xhochy/drossellog/logline.rb and try to correct it, if it
      # is really something missing, please report this as a bug.
      def parse
        path = File.join(@tmp_path, File.basename(@filename));
        path2 = File.join(path, 'sha1sum')
        
        File.open(@filename, 'r') do |f|
          @lines = []
          
          while line = f.gets
            if line =~ XhochY::Drossellog::LogLine.get_rxp
              line_obj = XhochY::Drossellog::LogLine.new(line)
              @lines << line_obj
            else # parse line
              puts line
            end
          end
        end
      end #^ parse
    
      # Save the data into sperate YAML-Files
      # 
      # The filenames consist of the following: <tmp_path>/<year>/<month>/<day>.yml
      def save_data(domain, domain_alias)
        sorted = {}
        @lines.each do |line|
          if sorted[line.year] == nil
            sorted[line.year] = {}
          end
          
          if sorted[line.year][line.month] == nil
            sorted[line.year][line.month] = {}
          end
          
          if sorted[line.year][line.month][line.day] == nil
            sorted[line.year][line.month][line.day] = []
          end
          
          sorted[line.year][line.month][line.day] << line
        end #^ @lines.each
        
        sorted.each do |year, value|
          value.each do |month, value2|
            value2.each do |day, value3|
              path = File.join(@tmp_path, File.basename(@filename), year.to_s, month)
              filename = File.join(path, day.to_s) + '.yml'
              FileUtils.mkdir_p(path)

              outdoc = {
              	'day' => day.to_i,
              	'month' => month,
              	'year' => year.to_i,
              	'domain' => domain,
              	'alias' => domain_alias,
              	'lines' => []
              }

              strip_tags = ['day', 'month', 'year']
              value3.each do |line|
                hash = line.to_hash strip_tags
                outdoc['lines'] << hash
              end
              
              File.open(filename, 'w') do |f|
                f.write outdoc.to_yaml
              end

            end #^ value2.each
          end #^ value.each
        end #^ sorted.each
    
        path = File.join(@tmp_path, File.basename(@filename), 'sha1sum')
        File.open(path, 'w') do |f|
          f.write(@sha1sum)
        end
      end #^save_data
      
    end

  end
  
end
