require 'xml/libxml'

module XhochY
  
  module Drossellog
  
    # Represents a line in a Apache combined-style logfile
    class LogLine
      # The whole line
      attr_reader :line
      # The IP which made the request
      attr_reader :ip
      # The day where the request was made 
      attr_reader :day
      # The month where the request was made
      attr_reader :month
      # The year where the request was made
      attr_reader :year
      # The hour where the request was made
      attr_reader :hour
      # The minute where the request was made
      attr_reader :minute
      # The second where the request was made
      attr_reader :second
      # The timezone of the server at the time of the request
      attr_reader :timezone
      # The method used to make the request
      attr_reader :method
      # The requested path
      attr_reader :path
      # The answered HTTP code form the server
      attr_reader :http_code
      # The size of the content sent back to the client
      attr_reader :size
      # The referrer which took the client to this point
      attr_reader :referrer
      # The requesting user agent
      attr_reader :agent
      
      # The regular expression to parse a apache logfile line
      @@rxp = /(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\s([^\s]+)\s([^\s]+)\s\[(\d{1,2})\/([^\/]+)\/(\d{4})\:(\d{1,2})\:(\d{1,2})\:(\d{1,2})\s([+-]\d{4})\]\s\"([A-Z]+)\s([^\s]+)[^\"]+\"\s(\d+)\s(\d+|-)\s\"([^\"]*)\"\s\"([^\"]*)\"/
    
      # check if the line is a valid apache-combined logfile line
      def self.matches?(line)
        return ('line' =~ @@rxp)
      end
      
      # return the regular expression for parsing a apache-combined logfile line
      def self.get_rxp
        @@rxp
      end
      
      # Create a new instance of a XYLogLine object
      def initialize(line)
        @line = line
        matches = @@rxp.match(line)
        @ip = matches[1]
        # matches[2] -> user ..
        # matches[3] -> user ..
        @day = matches[4].to_i
        @month = matches[5]
        @year = matches[6].to_i
        @hour = matches[7].to_i
        @minute = matches[8].to_i
        @second = matches[9].to_i
        @timezone = matches[10]
        @method = matches[11]
        @path = matches[12] 
        @http_code = matches[13]
        @size = matches[14].to_i
        @referrer = matches[15]
        @agent = matches[16]
      end
      
      # Return this object as a (Lib)XML-Node
      # 
      # In strip_tags we define which information should not be 
      # included into the node
      def to_xml_node(strip_tags)
        node = XML::Node.new('line')
        
        if !strip_tags.include?('ip')
          node << ip_node = XML::Node.new('ip')
          ip_node << @ip
        end
        
        if !strip_tags.include?('day')
          node << day_node = XML::Node.new('day')
          day_node << @day
        end
        
        if !strip_tags.include?('month')
          node << month_node = XML::Node.new('month')
          month_node << @month
        end
        
        if !strip_tags.include?('year')
          node << year_node = XML::Node.new('year')
          year_node << @year
        end
        
        if !strip_tags.include?('hour')
          node << hour_node = XML::Node.new('hour')
          hour_node << @hour
        end
        
        if !strip_tags.include?('minute')
          node << minute_node = XML::Node.new('minute')
          minute_node << @minute
        end
        
        if !strip_tags.include?('second')
          node << second_node = XML::Node.new('second')
          second_node << @second
        end
        
        if !strip_tags.include?('timezone')
          node << timezone_node = XML::Node.new('timezone')
          timezone_node << @timezone
        end
        
        if !strip_tags.include?('method')
          node << method_node = XML::Node.new('method')
          method_node << @method
        end
        
        if !strip_tags.include?('path')
          node << path_node = XML::Node.new('path')
          path_node << @path
        end
        
        if !strip_tags.include?('http_code')
          node << http_code_node = XML::Node.new('http_code')
          http_code_node << @http_code
        end
        
        if !strip_tags.include?('size')
          node << size_node = XML::Node.new('size')
          size_node << @size
        end
        
        if !strip_tags.include?('referrer')
          node << referrer_node = XML::Node.new('referrer')
          referrer_node << @referrer
        end
        
        if !strip_tags.include?('agent')
          node << agent_node = XML::Node.new('agent')
          agent_node << @agent
        end
        
        return node
      end
      
      # Returns this line as a Hash object
      def to_hash(strip_tags)
        hash = {}
        
        if !strip_tags.include?('ip')
          hash['ip'] = @ip
        end
        
        if !strip_tags.include?('day')
          hash['day'] = @day
        end
        
        if !strip_tags.include?('month')
          hash['month'] << @month
        end
        
        if !strip_tags.include?('year')
          hash['year'] = @year
        end
        
        if !strip_tags.include?('hour')
          hash['hour'] = @hour
        end
        
        if !strip_tags.include?('minute')
          hash['minute'] = @minute
        end
        
        if !strip_tags.include?('second')
          hash['second'] = @second
        end
        
        if !strip_tags.include?('timezone')
          hash['timezone'] = @timezone
        end
        
        if !strip_tags.include?('method')
          hash['method'] = @method
        end
        
        if !strip_tags.include?('path')
          hash['path'] = @path
        end
        
        if !strip_tags.include?('http_code')
          hash['http_code'] = @http_code
        end
        
        if !strip_tags.include?('size')
          hash['size'] = @size
        end
        
        if !strip_tags.include?('referrer')
          hash['referrer'] = @referrer
        end
        
        if !strip_tags.include?('agent')
          hash['agent'] = @agent
        end
        
        return hash
      end #^ to_hash
    end
    
  end
  
end
